@@ -9,7 +9,7 @@ import shutil
 cache_dir = user_cache_dir('pip') + "/roboflow"
 
 class RoboflowOak:
-    def __init__(self, model, version, api_key, confidence=0.2, overlap=0.5, rgb=False, depth=False, device=None, device_name="device", blocking=True, legacy=False):
+    def __init__(self, model, version, api_key, confidence=0.2, overlap=0.5, sensor_mode="THE_1080_P", wide_fov=False, rgb=False, depth=False, device=None, device_name="device", blocking=True, legacy=False):
         self.project = model
         self.version = version
         self.api_key = api_key
@@ -35,8 +35,9 @@ class RoboflowOak:
             self.resolution = self.size
             self.class_names = self.model_objects["class_names"]
             self.colors = self.model_objects["colors"]
+            print("Size",self.size,"Resolution",self.resolution)
         try:
-            self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, False, depth, device, legacy)
+            self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, sensor_mode, wide_fov, depth, device, legacy)
         except:
             try:
                 shutil.rmtree(cache_dir)
@@ -47,7 +48,7 @@ class RoboflowOak:
                     self.resolution = self.size
                     self.class_names = self.model_objects["class_names"]
                     self.colors = self.model_objects["colors"]
-                self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, False, depth, device, legacy)
+                self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, sensor_mode, wide_fov, depth, device, legacy)
             except:
                 raise Exception("Failure while retrying load weights - does this model, version, api key exist? can you curl api.roboflow.com, and can your device download files from google cloud storage? have you hit your device limit?")
 
@@ -58,11 +59,8 @@ class RoboflowOak:
 
         return download_blob(self.project, self.version, self.api_key, self.dev)
 
-    def detect(self, visualize=True):
-        if self.blocking:
-            ret = self.dai_pipe.get()         
-        else:
-            ret = self.dai_pipe.try_get()
+    def detect(self):
+        ret = self.dai_pipe.get()
 
         predictions = []
         dets = ret[0]
@@ -76,9 +74,12 @@ class RoboflowOak:
         ret[0] = {"predictions": predictions}
         return ret
 
+    def camera_control(self, control):
+        self.dai_pipe.camera_control(control)
+
 class OAKThread(threading.Thread, RoboflowOak):
-    def __init__(self, threadID, name, counter, onDetect, project, version, api_key, confidence=0.5, overlap=0.5, rgb=True, depth=False, device=None, device_name="device", blocking=True, legacy=False):
-        OAK.__init__(self, project, version, api_key, confidence, overlap, rgb, depth, device, device_name, blocking, legacy)
+    def __init__(self, threadID, name, counter, onDetect, project, version, api_key, confidence=0.5, overlap=0.5, sensor_mode="THE_1080_P", wide_fov=False, rgb=True, depth=False, device=None, device_name="device", blocking=True, legacy=False):
+        RoboflowOak.__init__(self, project, version, api_key, confidence, overlap, sensor_mode, wide_fov, rgb, depth, device, device_name, blocking, legacy)
         threading.Thread.__init__(self)
         self.threadID = threadID
         self.name = name
