@@ -2,10 +2,11 @@ import depthai as dai
 import numpy as np
 import cv2
 import roboflowoak.postprocs as postprocs
+import threading
 
 
 class DepthAIPipeline:
-    def __init__(self, nn_path, size, resolution, class_names, cam_stream, colors, confidence=0.5, overlap=0.5, stretch=False, depth=False, device=None, legacy=False):
+    def __init__(self, nn_path, size, resolution, class_names, cam_stream, colors, confidence=0.5, overlap=0.5, sensor_mode="THE_1080_P", stretch=False, depth=False, device=None, legacy=False):
         self.nn_path = nn_path
         self.size = size
         self.class_names = class_names
@@ -17,15 +18,27 @@ class DepthAIPipeline:
         self.depth = depth
         self.overlap = overlap
         self.confidence = confidence
+        self.frames = []
 
         self.pipeline = dai.Pipeline()
         if legacy:
             self.pipeline.setOpenVINOVersion(dai.OpenVINO.Version.VERSION_2021_1)
 
-        self.cam_rgb = self.pipeline.createColorCamera()
-        self.cam_rgb.setPreviewSize(self.resolution)
+        self.cam_rgb = self.pipeline.create(dai.node.ColorCamera)
+        self.cam_rgb.setBoardSocket(dai.CameraBoardSocket.RGB)
+        self.cam_rgb.setResolution(getattr(dai.ColorCameraProperties.SensorResolution, sensor_mode))
+        self.cam_rgb.setColorOrder(dai.ColorCameraProperties.ColorOrder.RGB)
         self.cam_rgb.setInterleaved(False)
 
+        self.xout_raw = self.pipeline.create(dai.node.XLinkOut)
+        self.xout_raw.setStreamName("raw")
+        self.xout_raw.input.setBlocking(False)
+        self.xout_raw.input.setQueueSize(1)
+
+        self.controlIn = self.pipeline.create(dai.node.XLinkIn)
+        self.controlIn.setStreamName('control')
+        self.controlIn.out.link(self.cam_rgb.inputControl)
+
         self.detection_nn = self.pipeline.createNeuralNetwork()
         try:
             self.detection_nn.setBlobPath(self.nn_path)
@@ -36,22 +49,32 @@ class DepthAIPipeline:
         if cam_stream:
             self.xout_rgb = self.pipeline.createXLinkOut()
             self.xout_rgb.setStreamName("rgb")
-            self.cam_rgb.preview.link(self.xout_rgb.input)
+            #self.cam_rgb.video.link(self.xout_rgb.input)
 
         self.xout_nn = self.pipeline.createXLinkOut()
         self.xout_nn.setStreamName("nn")
         self.detection_nn.out.link(self.xout_nn.input)
 
+
+        self.manip = self.pipeline.createImageManip()
+
+        self.manip.initialConfig.setCropRect((0,0,1,1))
+        self.manip.initialConfig.setResize(self.size)
+
+        self.manip_out = self.pipeline.createXLinkOut()
+        self.manip_out.setStreamName("manip")
+        self.manip.out.link(self.manip_out.input)
+
         if self.stretch:
-            self.manip = self.pipeline.createImageManip()
+            self.manip.initialConfig.setKeepAspectRatio(False)
+        self.manip.inputImage.setBlocking(True)
+        self.manip.initialConfig.setFrameType(dai.ImgFrame.Type.BGR888p)
 
-            self.manip.initialConfig.setResize(self.size)
-            self.manip.inputImage.setBlocking(True)
-
-            self.manip.out.link(self.detection_nn.input)
-            self.cam_rgb.preview.link(self.manip.inputImage)
-        else:
-            self.cam_rgb.preview.link(self.detection_nn.input)
+        self.manip.out.link(self.detection_nn.input)
+        self.cam_rgb.video.link(self.manip.inputImage)
+        if cam_stream:
+            self.cam_rgb.video.link(self.xout_raw.input)
+            self.manip.out.link(self.xout_rgb.input)
 
         if self.depth:
             self.left = self.pipeline.create(dai.node.MonoCamera)
@@ -100,8 +123,16 @@ class DepthAIPipeline:
 
         if cam_stream:
             self.q_rgb = self.device.getOutputQueue(name="rgb", maxSize=4, blocking=False)
+            self.q_raw = self.device.getOutputQueue(name="raw", maxSize=4, blocking=False)
+
+        self.q_manip = self.device.getOutputQueue(name="manip", maxSize=4, blocking=False)
+
+        frame_thread = threading.Thread(target=collect_frame_thread, args=(self,))
+        frame_thread.start()
 
     def disparity_to_depth(self, disparity):
+        if disparity == 0:
+            return np.NaN
         return 441.25 * 7.5 / disparity
 
     def detection_depth(self, detections, depth):
@@ -124,7 +155,28 @@ class DepthAIPipeline:
 
         return res
 
+    def camera_control(self, control):
+        q_control = self.device.getInputQueue(name="control")
+        q_control.send(control)
+
+
+    def print_seq_nums(self):
+        nums = [frame.getSequenceNum() for frame in self.frames]
+        print("Current Frames: ", nums)
+    def find_frame(self, in_det):
+        for i, frame in enumerate(self.frames):
+            if frame.getSequenceNum() == in_det.getSequenceNum():
+                self.frames = self.frames[i:]
+                return frame
+
+
+
     def get(self):
+        if self.depth:
+            in_depth = self.q_depth.get()
+
+        in_manip = self.q_manip.get()
+
         in_det = self.q_det.get()
 
         detections = self.post_processing(in_det)
@@ -132,52 +184,37 @@ class DepthAIPipeline:
         depth = None
 
         if self.depth:
-            in_depth = self.q_depth.get()
             depth = in_depth.getFrame()
             detections = self.detection_depth(detections, depth)
 
 
         if self.cam_stream:
-            in_rgb = self.q_rgb.get()
-            frame, frame_raw = self.process_frame(detections, in_rgb)
+            frame, frame_raw = self.process_frame(detections, in_det)
             return detections, frame, frame_raw, depth
 
         return detections, depth
 
-    def try_get(self):
-        in_det = self.q_det.tryGet()
-
-        detections = self.post_processing(in_det)
-
-        depth = None
-
-        if self.depth:
-            in_depth = self.q_depth.get()
-            depth = in_depth.getFrame()
-            detections = self.detection_depth(detections, depth)
-
-        if self.cam_stream:
-            in_rgb = self.q_rgb.tryGet()
-            frame, frame_raw = self.process_frame(detections, in_rgb)
-            return detections, frame, frame_raw, depth
-
-        return detections, depth
-
-    def process_frame(self, detections, in_rgb):
+    def process_frame(self, detections, in_det):
         frame_raw = None
         frame = None
-        if in_rgb is not None:
-            shape = (3, in_rgb.getHeight(), in_rgb.getWidth())
-            frame = in_rgb.getData()
-            frame = frame.reshape(shape)
-            frame = frame.transpose(1, 2, 0)
-            frame = frame.astype(np.uint8)
-            frame = np.ascontiguousarray(frame)
-            frame_raw = np.array(frame)
+        in_raw = self.find_frame(in_det)
+        if not in_raw:
+            return np.array([]), np.array([])
 
+        frame = in_raw.getCvFrame()
+        x_offset = int((in_raw.getWidth()-in_raw.getHeight())/2)
+        x_max = in_raw.getWidth()-x_offset
+
+        #frame = frame.reshape(shape)
+        #frame = frame.transpose(1, 2, 0)
+        frame = frame.astype(np.uint8)
+        frame = np.ascontiguousarray(frame)
+        if not self.stretch:
+            frame = frame[:,x_offset:x_max,:]
+        frame_raw = np.array(frame)
 
         if frame is not None:
-            for detection in self.scale_detections(detections):
+            for detection in self.scale_detections(detections, (in_raw.getWidth(), in_raw.getHeight())):
                 class_color = self.colors[detection[4]].strip("#")
                 class_color = tuple(int(class_color[i:i + 2], 16) for i in (0, 2, 4))
                 cv2.rectangle(frame, (detection[0], detection[1]), (detection[2], detection[3]), class_color, 2)
@@ -188,9 +225,12 @@ class DepthAIPipeline:
 
         return frame, frame_raw
 
-    def scale_detections(self, detections):
-        sx = self.resolution[0]/self.size[0]
-        sy = self.resolution[1]/self.size[1]
+    def scale_detections(self, detections, frame_size):
+
+        sx = frame_size[0]/self.size[0]
+        sy = frame_size[1]/self.size[1]
+        if not self.stretch:
+            sx = sy
         res = []
         for det in detections:
             res.append([
@@ -218,6 +258,12 @@ class DepthAIPipeline:
 
         return detections
 
+def collect_frame_thread(self):
+    while True:
+        if self.cam_stream:
+            in_rgb = self.q_rgb.get()
+            in_raw = self.q_raw.get()
+            self.frames.append(in_raw)
 def list_devices():
     available_devices = []
     for device in dai.Device.getAllAvailableDevices():
